package Security::passwd;

use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT = qw( &init() &check() &print() );

our (%uid0, %nopass, %dupuid, %defaultpw, %duppasswd, %oldpass);

sub init {
	print ( STDERR  "In Security::passwd::init()\n") if ( $main::opts{'d'} );

	&main::read_config('ignorepasswd');
	&main::read_config('ignoreuid');
	&main::read_config('ignoredupuid');
	&main::read_config('ignoreduppasswd');
	&main::read_config('defaultpass');
}

sub check {
	print ( STDERR "In Security::password::check()\n") if ( $main::opts{'d'} );

	my ($oldFld1, $olduid);
	$olduid = -5;
	setpwent;
	# FIXME: Need to sort based on the uid ($Fld3) field
	while (my ($Fld1,$Fld2,$Fld3,$Fld4) = getpwent() ) {

		# Check to see who has uids of 0.  This comes from the security 
		# script from BSD 4.4.  We do not need to report the uid 0 on 
		# the 'root' account, nor accounts that start with '+'
		$uid0{$Fld1} = 1 
			if (! $Fld3 && $Fld1 ne 'root' && $Fld1 ne '+' && 
				!grep (/^${Fld1}$/, @main::ignoreuid));

		# Check to see who has accounts without passwords.  This comes 
		# from the security script from BSD 4.4.  Ignore the accounts
		# that start with '+'
		$nopass{$Fld1} = 1 if ($Fld2 eq '' && $Fld1 ne '+' && 
				!grep (/^${Fld1}$/, @main::ignorepasswd));

		# Check to see who has accounts with duplicate passwords.  
		if ( $Fld2 !~ /(^(x|NP|!+)?$)|\*/ ) {
			if ( exists $oldpass{$Fld2} ) {
				$duppasswd{$Fld1} = 'and ' . $oldpass{$Fld2}
					if (!grep (/^${Fld1}$/, @main::ignoreduppasswd));
				$oldpass{$Fld2} .= ' ' . $Fld1
			} else {
				$oldpass{$Fld2} = $Fld1;
			}
		}

		# Check to see who has accounts with default passwords.  The
		# admin will define the list of default encrypted passwords in
		# the config file.
		$defaultpw{$Fld1} = 1 if ($Fld1 ne '+' && 
				scalar @main::defaultpass &&
				grep (/^\Q${Fld2}\E$/, @main::defaultpass));

		# Check to see what accounts have duplicate uid's (except the 
		# root account, which was handled earlier.  This idea comes from 
		# the HP-UX System Security manual
		$dupuid{$Fld1} = $oldFld1 if ( $olduid eq $Fld3 && $Fld3  &&
				!grep (/^${Fld1}$/, @main::ignoredupuid));

		$olduid = $Fld3;
		$oldFld1 = $Fld1;
	}
	endpwent;
}

sub print () {
	print ( STDERR  "In Security::passwd::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking for uids of 0:\n", %uid0);
	&main::print_data ("\n\nchecking for accounts with no passwords:\n", %nopass);
	&main::print_data ("\n\nchecking for accounts with default passwords:\n", %defaultpw);
	&main::print_data ("\n\nchecking for duplicate uids:\n", %dupuid);
	&main::print_data ("\n\nchecking for duplicate passwords:\n", %duppasswd);
}

1;
