package Security::rootlogin;

use Exporter;
use strict;
use warnings;
no warnings 'once';

our @ISA = qw(Exporter);
our @EXPORT = qw( &init() &check_ttys() &check_access() &check_defaultlogin() &print() );

our (%ttys);

sub init {
	print ( STDERR "In Security::rootlogin::init()\n") if ( $main::opts{'d'} );

	# Read the config file for root port definitions.
	&main::read_config('ignorerootports');

	$main::rootlogin = -1;
}

sub check_securettys {
	local *FP;

	print ( STDERR "In Security::rootlogin::check_securettys()\n") if ( $main::opts{'d'} );

	open (FP, $main::securefile) || 
		die "Could not open login file $main::securefile\n";
	grep (/^(.*)$/o && ($ttys{$1}=1), <FP>);
	close FP;

	&_remove_defaultports();
}

sub check_defaultlogin {
	local *FP;

	print ( STDERR "In Security::rootlogin::check_defaultlogin()\n") if ( $main::opts{'d'} );
	open (FP, $main::loginfile) || die "Could not open login file $main::loginfile\n";
	$main::rootlogin=grep (/^CONSOLE/o, <FP>);
	close FP;

	&_remove_defaultports();
}

sub check_ttys {
	local *FP;

	printf STDERR "In Security::rootlogin::check_ttys()\n"
		if ( $main::opts{'d'} );
	open (FP, $main::securefile) || 
		die "Could not open login file $main::securefile\n";
	grep (/^([^ \t#]+)[ \t]*.*[ \t]*on[ \t]*secure/o && ($ttys{$1}=1), <FP>);
	close FP;

	&_remove_defaultports();
}

sub check_access {
	local *FP;

	print ( STDERR "In Security::rootlogin::check_access()\n") if ( $main::opts{'d'} );
	open (FP, $main::accessfile) || 
		die "Could not open login file $main::accessfile\n";
	while (<FP>) {
		my (@a) = split (/:/o, $_) if ( $_ !~ /^#/o );
		if ( $a[1] eq "root" ) {
			if ( $a[0] eq "+" ) {
				if ( $a[2] =~ /ALL$/o ) {
					$main::rootlogin=0;
				} else {
					my (@b) = split (/[ \t]+/o, $a[2]);
					for (my ($i) = 0; $i < $#b; $i++) {
						$ttys{$b[$i]} = 1 if ( $b[$i] ne 'ALL' );
					}
				}
			} else {
				if ( $a[2] =~ /ALL$/o ) {
					$main::rootlogin=1;
				} else {
					my (@b) = split (/[ \t]+/o, $a[2]);
					for (my ($i) = 0; $i < $#b; $i++) {
						$ttys{$b[$i]} = 1 
							if ( $b[$i] ne 'ALL' && 
								$b[$i] ne 'EXCEPT' );
					}
				}
			}
		}
	}
	close FP;

	&_remove_defaultports();
}

sub _remove_defaultports {
	# Now remove the preconfigured root ports
	for (my ($i) = 0; $i < scalar (@main::ignorerootports); $i++) {
		if ( defined $ttys{$main::ignorerootports[$i]} ) {
			printf STDERR "unsetting $main::ignorerootports[$i] in ttys array\n"
				if ( $main::opts{'d'} );
			delete $ttys{$main::ignorerootports[$i]};
		}
	}
}

sub print {
	printf STDERR "In Security::rootlogin::print()\n" if ( $main::opts{'d'} );

	if ( ! $main::rootlogin ) {
		&main::print_header("\n\nRoot login allowed on all ports\n");
     }

	&main::print_data ("\n\nchecking for ports root is allowed to login to:\n", %ttys);
}

1;
